--[[
 <Dispell Alert : This addon allows you to alert you when there is a spell that affects you yourself!>
    Copyright (C) <2018>  <Caron Sébastien>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see http://www.gnu.org/licenses.
]]
--manspace
local _, ns = ... 
local uiFrame = ns.uiFrame
local uiTarget = ns.uiTarget

--variables
local bdMax = 40
local dispellable = {"Disease","Poison","Magic","Curse"}
local index = nil
local indexNo = nil
local tempNameDeBuff = nil
local tempNameDeBuffNo = nil
local isWaitDispell = false
local isNoSpecDispell = false
local dispell = {
	[11] = {"Poison","Curse"},
	[8] = {"Curse"},
	[2] = {"Disease","Poison"},
	[10] = {"Disease","Poison"},
	[5] = {"Magic"},
	[7] = {"Curse"},
	[9] = {"Magic"}
}
local specialization = {105,70,68,264,268,256,257}
local dispellSpec = {
	[11] = {"Poison","Curse","Magic"},
	[2] = {"Disease","Poison","Magic"},
	[10] = {"Disease","Poison","Magic"},
	[5] = {"Magic","Disease"},
	[7] = {"Curse","Magic"}
}
--function
local function in_array(tab,matcher)
	if tab == nil or matcher == nil then return false end
	for i = 1, #tab do
		if tab[i] == matcher then
			return true
		end
	end
	return false
end

local function priority(duration)
	local str = "LOW"
	if duration >= tonumber(ns.config.priority[1]) and duration < tonumber(ns.config.priority[2]) then
		str = "MEDIUM"
	elseif duration >= tonumber(ns.config.priority[2]) then
		str = "HIGH"
	end	
	return str
end

local function showFrame(self,name,caster,icon)
	if ns.config.ui then
		local nameEnemy = "No Information"
		if UnitExists(caster) then
			nameEnemy = GetUnitName(caster,false)
		end
		self.nameDispell:SetText(name)
		self.nameCasterDispell:SetText(nameEnemy)
		self.icon:SetTexture(icon)
		if ns.config.sound then
			PlaySound(12889, "Master");
		end
		self:Show()
	end
end

local function showMessage(name,type,duration, spellID)
	--name debuff not null
	if ns.config.msg then
		if duration ~= nil then
			if in_array(dispellable,type) then
				local link = GetSpellLink(spellID)
				if priority(duration) == "LOW" then
					DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r dispell priority : |cff66ff00("..priority(duration)..")")
					DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r name spell : "..link.." Type : ("..type..")!")
				elseif priority(duration) == "MEDIUM" then
					DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r dispell priority : |cffffff00("..priority(duration)..")")
					DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r name spell : "..link.." Type : ("..type..")!")
				else
					DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r dispell priority : |cffff0000("..priority(duration)..")")
					SendChatMessage("Need Dispell "..link.." Type : ("..type..")!","SAY","COMMON",ChatFrame1)
				end
				if ns.config.sound then
					PlaySound(15273, "Master");
				end
				isNoSpecDispell = true
				tempNameDeBuffNo = name
			end
		end
	end
end


--function relative On Event
uiFrame:SetScript("OnEvent",function (self, event, id)
	if id == "player" then 
		local spec = GetSpecialization()
		local infoSpecId, infoSpecName = GetSpecializationInfo(spec,false,false,false,UnitSex(id))
		local infoName = UnitName(id)
		local infoClass, _, classID = UnitClass(id)
		for i = 1, bdMax do
			local deBuff, icon ,count ,debuffType ,duration, _, unitCaster,
			isStealable, _, spell = UnitDebuff(id,i)
			
			if deBuff ~= tempNameDeBuff and isWaitDispell  and i == index then
				tempNameDeBuff = nil
				isWaitDispell = false
				index = nil
				self:Hide()
				break
			else
				if in_array(specialization,infoSpecId) then
					if in_array(dispellSpec[classID],debuffType) then
						if isWaitDispell then return end
						showFrame(self,deBuff,unitCaster,icon)
						tempNameDeBuff = deBuff
						index = i
						isWaitDispell = true
					else
						if not isNoSpecDispell then
							showMessage(deBuff,debuffType,duration,spell)
							indexNo = i
						end
						if deBuff ~= tempNameDeBuffNo and isNoSpecDispell and i == indexNo then
							isNoSpecDispell = false
							tempNameDeBuffNo = nil
							indexNo = nil
						end
					end
				else
					if in_array(dispell[classID],debuffType) then
						if isWaitDispell then return end
						showFrame(self,deBuff,unitCaster,icon)
						tempNameDeBuff = deBuff
						index = i
						isWaitDispell = true
					else
						if not isNoSpecDispell then
							showMessage(deBuff,debuffType,duration,spell)
							indexNo = i
						end
						if deBuff ~= tempNameDeBuffNo and isNoSpecDispell and i == indexNo then
							isNoSpecDispell = false
							tempNameDeBuffNo = nil
							indexNo = nil
						end
					end
				end
			end
		end
	end
end)
