--[[
 <Dispell Alert : This addon allows you to alert you when there is a spell that affects you yourself!>
    Copyright (C) <2018>  <Caron Sébastien>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see http://www.gnu.org/licenses.
]]
--namesapce
local _, ns = ...
-------------
local config = {} 
local loading = false
local mf = false
local uiFrame, uiTarget = nil, nil

local function config_a()
	if Dispell_AlertDB == nil then
		Dispell_AlertDB = {}
		Dispell_AlertDB.sound = true
		Dispell_AlertDB.msg = true
		Dispell_AlertDB.ui = true
		Dispell_AlertDB.priority = {9,30} -- default value
	end
	config = Dispell_AlertDB
	ns.config = config
	uiFrame = ns.uiFrame
	uiTarget = ns.uiTarget
	if IsAddOnLoaded("Dispell_Alert") then
		DEFAULT_CHAT_FRAME:AddMessage("Dispell Alert loaded, cmd : /da,  (version : 8.0.7)", 0.6, 0.3, 0.8)
	end
end

local function help()
	if loading then
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r ==========================")
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r |cff66ff00/da sound :|r on/off sound alert !")
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r |cff66ff00/da showmsg :|r show/hide message alert in chat !")
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r |cff66ff00/da showui :|r show/hide user interface alert !")
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r |cff66ff00/da move :|r move user interface alert !")
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r |cff66ff00/da priority max (value) :|r this value must be an integer, for the message display priority ")
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r |cff66ff00/da priority min (value) :|r this value must be an integer, for the message display priority ")
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r ==========================")
	end
end

local function priorityValue(typevalue,value)
	if typevalue == "max" then
		if tonumber(value) >= 12 then
			config.priority[2] = value
		else
			DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r caution value not compatible lower than 12 !")
		end
		--complite
	end
	if typevalue == "min" then
		if tonumber(value) >= 2 and tonumber(value) <= 9 then
			config.priority[1] = value
		else
			DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r caution value not compatible lower than 2 or greater than 9 !")
		end
		--complite
	end
end

local function move(frame)
	--local tex = frame:CreateTexture(nil,"BACKGROUND")
	--tex:SetAllPoints()
	if mf then
		frame:Show()
		frame:SetScript("OnDragStart",uiFrame.StartMoving)
		frame:SetScript("OnDragStop",uiFrame.StopMovingOrSizing)
		--tex:SetColorTexture(0,0,0,0.50)
		
	else
		--tex:SetColorTexture(0,0,0,0.0)
		frame:Hide()
	end
end

local function frameMove()
	local info = nil
	if mf then
		mf = false
		move(uiFrame)
		move(uiTarget)
		info = "off"
	else
		mf = true
		move(uiFrame)
		move(uiTarget)
		info = "on"
	end
	if loading then
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r move set "..info.." !")
	end
end

local function sound_toggle()
	local info = nil
	if config.sound then
		config.sound = false
		info = "off"
	else
		config.sound = true
		info = "on"
	end
	if loading then
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r sound set "..info.." !")
		ns.config = config
	end
end

local function msg_toggle()
	local info = nil
	if config.msg then
		config.msg = false
		info = "hide"
	else
		config.msg = true
		info = "show"
	end
	if loading then
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r message set "..info.. " !")
		ns.config = config
	end
end

local function ui_toggle()
	local info = nil
	if config.ui then
		config.ui = false
		info = "hide"
	else
		config.ui = true
		info = "show"
	end
	if loading then
		DEFAULT_CHAT_FRAME:AddMessage("Dispell |cff9966ffAlert : |r ui set "..info.." !")
		ns.config = config
	end
end

local function slashConfig(msg)
	if msg ~= nil then
		if msg == "sound" then
			sound_toggle()
		elseif msg == "showmsg" then
			msg_toggle()
		elseif msg == "showui" then
			ui_toggle()
		elseif msg == "move" then
			frameMove()
		else
			if msg ~= "" then
				local args, other, value = strsplit(" ",msg)
				if args == "priority" then
					if other == "max" then
						priorityValue("max",value)
						print("max",value)
						return
					else
						help()
					end
					if other == "min" then
						priorityValue("min",value)
						print("min",value)
						return
					else
						help()
					end
				end
			else
				help()
			end
		end
	end
end

local function init(name)
	if name ~= "Dispell_Alert" then
		return
	end
	
	for i = 1 , NUM_CHAT_WINDOWS do
		_G["ChatFrame"..i.."EditBox"]:SetAltArrowKeyMode(false)
	end
	--commande slash
	SLASH_FRAMESTK1 = "/fs"
	SlashCmdList.FRAMESTK = function ()
		LoadAddOn('Blizzard_DebugTools')
		FrameStackTooltip_Toggle()
	end

	--load addon
	SLASH_Dispell_Alert1 = "/da"
	SlashCmdList["Dispell_Alert"] = slashConfig
	loading = true
	ns.config = config
end


local frame = CreateFrame("Frame")
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("PLAYER_LOGIN")
frame:RegisterEvent("PLAYER_LOGOUT")
frame:SetScript("OnEvent",function(self,event,name)
	if event == "ADDON_LOADED" then
		init(name)
	elseif event == "PLAYER_LOGIN" then
		config_a()
	elseif event == "PLAYER_LOGOUT" then
		Dispell_AlertDB = config
	end
end)